/*
 * Main.c - Program to write a boot sector with a custom message onto a floppy
 *	disk.
 *
 *	MainProc
 *	WriteMsg
 *
 * Copyright 2002 William R. Swanson
 * site@swansontec.com
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "Main.h"

/******************************************************************************
 * Global Data:
 */

HINSTANCE hInstance;

/******************************************************************************
 * WinMain - Startup code.
 */

int WINAPI WinMain (HINSTANCE hInst, HINSTANCE hPrevious, LPSTR pstrCommand, int iCommand)
{
	hInstance = hInst;

	DialogBox(hInstance, MAKEINTRESOURCE(IDD_BOX), NULL, (DLGPROC) MainProc);

	return 1;
}

/******************************************************************************
 * MainProc - Window handler for the main dialog box.
 */

BOOL CALLBACK MainProc (HWND hwnd, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	LRESULT beep;
	char msg[MSG_LEN];
	HICON icon;

	switch (iMessage)
	{
	case WM_INITDIALOG:
		icon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON));
		SetClassLong(hwnd, GCL_HICON, (LONG)icon);
		return TRUE;
	
	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_WRITE:
			GetDlgItemText(hwnd, IDC_EDIT, msg, MSG_LEN);
			beep = SendDlgItemMessage(hwnd, IDC_BEEP,
				BM_GETCHECK, 0, 0);
			WriteMsg (beep == BST_CHECKED, msg);
			return TRUE;
		case IDC_EXIT:
			EndDialog (hwnd, 1);
			return TRUE;
		
		default:
			return FALSE;
		}
	
	case WM_SYSCOMMAND:
		switch (wParam)
		{
		case SC_CLOSE:
			EndDialog (hwnd, 1);
			return TRUE;
		default:
			return FALSE;
		}
	
	default:
		return FALSE;
	}
}

/******************************************************************************
 * WriteMsg - Writes the custom boot code and specified message to the disk.
 */

void WriteMsg (BOOL beep, char msg[])
{
	HRSRC hrcBC;
	HGLOBAL hBC;
	char *boot_code;
	unsigned char buffer[SECTOR_SIZE];
	int i;

	if(!SectorRead (buffer, 0, 1, 0))
		return;

	if((hrcBC = FindResource(NULL, MAKEINTRESOURCE(IDR_BOOTCODE), RT_RCDATA)) == NULL)
		vErrAPI(APP_NAME);
	if((hBC = LoadResource(NULL, hrcBC)) == NULL)
		vErrAPI(APP_NAME);
	if((boot_code = LockResource(hBC)) == NULL)
		vErrAPI(APP_NAME);

	buffer[0] = 0xEB;	/*jmp*/
	buffer[1] = 0x3E;	/*40h*/
	buffer[2] = 0x90;	/*nop*/
	for(i = 0; i < *((long *)boot_code); i++)
		buffer[0x40 + i] = boot_code[4 + i];
	while(*msg)
		buffer[0x40 + i++] = *(msg++);
	if(beep)
		buffer[0x40 + i++] = '\a';
	buffer[0x40 + i++] = '\0';
	buffer[0x1FE] = 0x55;
	buffer[0x1FF] = 0xAA;

	SectorWrite (buffer, 0, 1, 0);
	return;
}

